"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request");
var zlib = require("zlib");
var uuid = require("uuid/v1");
var sl_env_vars_1 = require("../constants/sl-env-vars");
var http_verb_1 = require("./http-verb");
var HttpClient = /** @class */ (function () {
    function HttpClient(cfg, logger) {
        this.cfg = cfg;
        this.logger = logger;
        if (!cfg) {
            throw new Error("cfg is required");
        }
        if (!logger) {
            throw new Error("logger is required");
        }
        this.defaultTimeout = this.getDefaultTimeout();
    }
    HttpClient.prototype.get = function (urlPath, callback) {
        this.invokeHttpRequest(http_verb_1.HttpVerb.GET, urlPath, callback);
    };
    HttpClient.prototype.delete = function (urlPath, callback) {
        this.invokeHttpRequest(http_verb_1.HttpVerb.DELETE, urlPath, callback);
    };
    HttpClient.prototype.post = function (requestData, urlPath, callback) {
        var _this = this;
        var bufferToSend = new Buffer(JSON.stringify(requestData));
        var shouldZip = this.cfg.compressRequests || true;
        this.logger.debug("Sending buffer:" + bufferToSend.toString());
        if (shouldZip) {
            zlib.gzip(bufferToSend, function (err, compressedBuf) {
                if (err) {
                    //Fallback to sending data uncompressed
                    _this.logger.warn("Failed while trying to compress the request data. Sending uncompressed data instead. Error: ", err);
                    _this.invokeHttpRequest(http_verb_1.HttpVerb.POST, urlPath, callback, null, bufferToSend);
                }
                else {
                    _this.invokeHttpRequest(http_verb_1.HttpVerb.POST, urlPath, callback, { "Content-Encoding": "gzip" }, compressedBuf);
                }
            });
        }
        else {
            this.invokeHttpRequest(http_verb_1.HttpVerb.POST, urlPath, callback, null, bufferToSend); //Submit buffer as-is
        }
    };
    HttpClient.prototype.invokeHttpRequest = function (httpVerb, urlPath, callback, additionalHeaders, buffer) {
        try {
            var opts = this.createDefaultOptions(urlPath);
            if (additionalHeaders) {
                for (var header in additionalHeaders) {
                    opts.headers[header] = additionalHeaders[header];
                }
            }
            var id = uuid();
            var onRequestCallback = this.createOnRequestCallback(httpVerb, id, callback);
            this.logger.info("Sending " + httpVerb + " request. Url: '" + opts.url + "', requestId: '" + id + "'");
            if (httpVerb === http_verb_1.HttpVerb.GET) {
                opts.json = true;
                request.get(opts, onRequestCallback);
            }
            else if (httpVerb === http_verb_1.HttpVerb.DELETE) {
                opts.json = true;
                request.delete(opts, onRequestCallback);
            }
            else if (httpVerb === http_verb_1.HttpVerb.POST) {
                opts.body = buffer;
                request.post(opts, onRequestCallback);
            }
            else {
                new Error(httpVerb + " is not implemented yet.");
            }
        }
        catch (err) {
            this.logger.error("Failed sending Http " + httpVerb + " request to:'" + urlPath + "'. Error: '" + err + "'");
            return callback(err, null);
        }
    };
    HttpClient.prototype.createOnRequestCallback = function (httpVerb, requestId, callback) {
        var _this = this;
        var handler = function (err, requestResponse, body) {
            if (err) {
                _this.logger.error("Had error in '" + httpVerb + "' request. Request id:'" + requestId + "', Error: " + err);
                return callback(err, null);
            }
            if (requestResponse.statusCode < 200 || requestResponse.statusCode >= 400) {
                _this.logger.warn("Got " + requestResponse.statusCode + " in '" + httpVerb + "' request. Request id:'" + requestId + "'.");
                var err_1 = requestResponse.statusCode == 404 ? null : new Error("Server returned HTTP Status " + requestResponse.statusCode);
                return callback(err_1, null);
            }
            if (httpVerb === http_verb_1.HttpVerb.POST) {
                body = requestResponse.body;
                if (body && body.length) {
                    body = JSON.parse(requestResponse.body);
                }
            }
            _this.logger.info("'" + httpVerb + "' request was completed successfully. Request id:'" + requestId + "'. statusCode: " + requestResponse.statusCode + " body: " + body);
            return callback(null, body);
        };
        return handler;
    };
    HttpClient.prototype.allowUntrustedCertificates = function () {
        //For more information: https://stackoverflow.com/questions/17383351/how-to-capture-http-messages-from-request-node-library-with-fiddler/22996076#22996076
        process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
    };
    HttpClient.prototype.createDefaultOptions = function (urlPath) {
        //get
        var opts = {
            url: this.cfg.server + urlPath,
            headers: {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + this.cfg.token
            },
            timeout: this.defaultTimeout
        };
        //TODO: Verify if we need it. Nadav, 21.12.18
        opts["compressed"] = true;
        if (this.cfg.proxy != null) {
            opts.proxy = this.cfg.proxy;
            this.allowUntrustedCertificates();
        }
        return opts;
    };
    HttpClient.prototype.getDefaultTimeout = function () {
        var timeout = sl_env_vars_1.SlEnvVars.getHttpTimeout();
        if (timeout == null) {
            timeout = 60 * 1000 * 2;
        }
        this.logger.debug("Http timeout value is '" + timeout + "'.");
        return timeout;
    };
    return HttpClient;
}());
exports.HttpClient = HttpClient;
